<?php

namespace SIU\SQ\Lib\procesos;

use SIU\SQ\Lib\errores\error;
use SIU\SQ\Lib\procesos\proceso;
use SIU\SQ\Lib\modelo\notificacion;
use SIU\SQ\Lib\constantes;
use SIU\SQ\Lib\lib;
use setasign\Fpdi;

class proceso_enviar_emails extends proceso
{
	private $ids_envios_notificaciones_fallados = array();
	private $ids_envios_notificaciones_exitosos = array();
	private $email_from_defecto = null;//esta variable la usa el mailer
	private $email_from_name = null;
	private $notificaciones_a_enviar = [];
	
	
	private $mensajes_de_error = "";	
	
	function get_ids_envios_notificaciones_fallados()
	{
		return $this->ids_envios_notificaciones_fallados;
	}

	function get_ids_envios_notificaciones_exitosos()
	{
		return $this->ids_envios_notificaciones_exitosos;
	}
	
	function set_notificaciones_a_enviar(array $notificaciones)
	{
		$this->notificaciones_a_enviar = $notificaciones;
	}
		
	public function ejecutar_especifico()
	{
		$this->set_estado(self::ESTADO_OK);
		try{
			if(empty($this->notificaciones_a_enviar)){
				$filtro = ['enviada_en__null' => true, 'notificado_a__not_null' => true];
				$this->notificaciones_a_enviar = lib::catalogo_daos()->notificaciones()->get_lista($filtro);
			}
			$mailer = lib::mailer();
			$this->email_from_defecto = lib::parametros_sistema()->get_valor(constantes::PARAMETROS_SISTEMA_CODIGO_EMAIL_SENDER);
			$this->email_from_name = lib::parametros_sistema()->get_valor(constantes::PARAMETROS_SISTEMA_CODIGO_EMAIL_FROM_NAME);
			$mailer->set_from($this->email_from_defecto, $this->email_from_name);
			foreach($this->notificaciones_a_enviar as $n){
				try{
					if(is_array($n)){
						$n = lib::catalogo_modelo()->notificacion($n['id']);
					}
					$this->enviar_email($mailer,$n);
					$this->ids_envios_notificaciones_exitosos[] = $n->get_id();
				} catch (\Exception $ex) {
					$this->ids_envios_notificaciones_fallados[] = $n->get_id();
					$this->mensajes_de_error .= $ex->getMessage() . PHP_EOL;
					$this->entorno()->log()->err($ex->getMessage());
					$this->agregar_notificacion($ex->getMessage(), 'error');
					$this->set_estado(self::ESTADO_ERROR);
				}
			}
			
			$this->log_info();
			
		} catch (error $e) {
			$this->agregar_notificacion($e->getMessage(), 'error');
			$this->finalizar_registro_ejecucion(proceso::ESTADO_ERROR);
			throw $e;
		}
		$this->finalizar_registro_ejecucion();
	}
	
	function enviar_email($mailer,notificacion $notificacion)
	{
		//parametros para mailer
		$destinatario_email = $notificacion->get_notificado_a();
		if(empty($destinatario_email)){
			throw new error(__METHOD__ . ' No se pudo obtener el email destinatario de la notificacion con ID: ' . $notificacion->get_id());
		}
		$asunto = $notificacion->get_tema();
		$cuerpo_email = $notificacion->get_mensaje();
			$this->entorno()->log()->err("................NOTIFICACION.................-\n" . $notificacion->get_id());
		if($notificacion->get_id_tipo_notificacion() == constantes::TIPO_NOTIFICACION_CUPON_PAGO){
			$venta = lib::catalogo_daos()->notificaciones()->get_lista(['id' => $notificacion->get_id()])[0];
			if(isset($venta['id_comprobante'])){
				$comprobante = lib::catalogo_daos()->comprobantes()->get($venta['id_comprobante']);
				$partes = explode('/', $comprobante['id_rdi']);
				$mailer->adjuntar_archivo($comprobante['id_rdi'], end($partes),  'base64', 'application/pdf');	
			}
		}
		$mailer->enviar_email($asunto,$cuerpo_email,$destinatario_email);

		$notificacion->set_enviada_en('now()');
		$notificacion->set_enviada_por($this->email_from_name);
		$notificacion->guardar();
	}
    
    public function generar_pdf(array $datos_venta, $nombre_comprobante_jasper, $datos_qr = null)
    {
        $generador_de_pdf = lib::toba_vista_jasperreports_json();
        $generador_de_pdf->set_datos_venta($datos_venta);
        $generador_de_pdf->set_path_reporte(\toba::proyecto()->get_path().'/../reportes/' . $nombre_comprobante_jasper);
        if ($datos_qr) {
            $generador_de_pdf->set_datos_qr($datos_qr);
        }
        $generador_de_pdf->generar_pdf();
        return $generador_de_pdf->get_nombre_archivo_generado();
    }
    
    public function generar_documento_unico($comprobantes_generados)
    {
        // https://manuals.setasign.com/fpdi-manual/v2/the-fpdi-class/examples/
        $pdf = new Fpdi\Fpdi();
        
        foreach ($comprobantes_generados as $file) {
            $pageCount = $pdf->setSourceFile($file);
            for ($pageNo = 1; $pageNo <= $pageCount; $pageNo++) {
                $templateId = $pdf->importPage($pageNo);
                $size = $pdf->getTemplateSize($templateId);
                $pdf->AddPage($size['orientation'], $size);
                $pdf->useTemplate($templateId);
            }
        }
        
        $archivo_salida = \toba::proyecto()->get_path_temp() . '/' . date('YmdHis') . '_factura' . '.pdf';
        
        $pdf->Output('F', $archivo_salida);
        
        return $archivo_salida;
    }
	
	private function log_info()
	{
		$log_info = "Emails enviados: " . count($this->ids_envios_notificaciones_exitosos) . PHP_EOL;
		if(count($this->ids_envios_notificaciones_exitosos) > 0){
			$log_info .= "IDs SQ notificaciones enviadas por email: " . implode(',',$this->ids_envios_notificaciones_exitosos) . PHP_EOL;
		}
		$log_info .= '--------------------------------------------------------------------' . PHP_EOL;
		$log_info .= "Envio de emails fallados: " . count($this->ids_envios_notificaciones_fallados) . PHP_EOL;
		if(count($this->ids_envios_notificaciones_fallados) > 0){
			$log_info .= "IDs SQ notificaciones de las que fallo el envio de email: " . implode(',', $this->ids_envios_notificaciones_fallados) . PHP_EOL;
			$log_info .= '--------------------------------------------------------------------' . PHP_EOL;
			$log_info .= "Errores: " . PHP_EOL . $this->mensajes_de_error . PHP_EOL;
		}
		
		//feedback en consola
		if($this->es_verboso()){
			echo $log_info;
		}
		$this->agregar_notificacion($log_info, 'info');
	}

}
